# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer

import plone.formwidget.hcaptcha


class PloneFormwidgetHcaptchaLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=plone.formwidget.hcaptcha)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "plone.formwidget.hcaptcha:default")


PLONE_FORMWIDGET_HCAPTCHA_FIXTURE = PloneFormwidgetHcaptchaLayer()


PLONE_FORMWIDGET_HCAPTCHA_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_FORMWIDGET_HCAPTCHA_FIXTURE,),
    name="PloneFormwidgetHcaptchaLayer:IntegrationTesting",
)



# -*- coding: utf-8 -*-
# Code taken from external dependency
# https://pypi.org/project/nohcaptcha/, which is not
# updated to Python 3
from six.moves.urllib import parse
from six.moves.urllib.request import Request
from six.moves.urllib.request import urlopen

import six


try:
    import json
except ImportError:
    import simplejson as json


VERIFY_SERVER = "hcaptcha.com"


class HcaptchaResponse(object):
    def __init__(self, is_valid, error_code=None):
        self.is_valid = is_valid
        self.error_code = error_code

    def __repr__(self):
        return "Hcaptcha response: {0} {1}".format(self.is_valid, self.error_code)

    def __str__(self):
        return self.__repr__()


def displayhtml(
    site_key, language="", theme="light", fallback=False, d_type="image", size="normal"
):
    """
    Gets the HTML to display for HCAPTCHA

    site_key -- The site key
    language -- The language code for the widget.
    theme -- The color theme of the widget. `light` or `dark`
    fallback -- Old version hcaptcha.
    d_type -- The type of CAPTCHA to serve. `image` or `audio`
    size -- The size of the dispalyed CAPTCHA, 'normal' or 'compact'

    """

    return """
<script
  src="https://hcaptcha.com/1/api.js?hl=%(LanguageCode)s&fallback=%(Fallback)s&"
  async="async" defer="defer"></script>
<div class="h-captcha"
    data-sitekey="%(SiteKey)s"
    data-theme="%(Theme)s"
    data-type="%(Type)s"
    data-size="%(Size)s">
</div>
""" % {
        "LanguageCode": language,
        "SiteKey": site_key,
        "Theme": theme,
        "Type": d_type,
        "Size": size,
        "Fallback": fallback,
    }


def submit(hcaptcha_response_field, secret_key, remoteip, verify_server=VERIFY_SERVER):
    """
    Submits a HCAPTCHA request for verification. Returns HcaptchaResponse
    for the request

    hcaptcha_response_field -- The value from the form
    secret_key -- your HCAPTCHA secret key
    remoteip -- the user's ip address
    """

    if not (hcaptcha_response_field and len(hcaptcha_response_field)):
        return HcaptchaResponse(is_valid=False, error_code="incorrect-captcha-sol")

    def encode_if_necessary(s):
        if isinstance(s, six.text_type):
            return s.encode("utf-8")
        return s

    if six.PY2:
        secret_key = encode_if_necessary(secret_key)
        remoteip = encode_if_necessary(remoteip)
        hcaptcha_response_field = encode_if_necessary(hcaptcha_response_field)

    params = parse.urlencode(
        {
            "secret": secret_key,
            "response": hcaptcha_response_field,
        }
    )

    request = Request(
        url="https://{0}/siteverify".format(verify_server),
        data=params,
        headers={
            "Content-type": "application/x-www-form-urlencoded",
            "User-agent": "HCAPTCHA Python",
        },
    )

    if six.PY3:
        request.data = request.data.encode("utf-8")

    httpresp = urlopen(request)

    return_values = json.loads(httpresp.read())
    httpresp.close()

    return_code = return_values["success"]
    error_codes = return_values.get("error-codes", [])

    if return_code:
        return HcaptchaResponse(is_valid=True)
    else:
        return HcaptchaResponse(is_valid=False, error_code=error_codes)



# -*- coding: utf-8 -*-
from plone.formwidget.hcaptcha.interfaces import IHCaptchaSettings
from plone.formwidget.hcaptcha.nohcaptcha import displayhtml
from plone.formwidget.hcaptcha.nohcaptcha import submit
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope import schema
from zope.annotation import factory
from zope.component import adapter
from zope.component import queryUtility
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest


class IHcaptchaInfo(Interface):
    error = schema.TextLine()
    verified = schema.Bool()


@adapter(IBrowserRequest)
@implementer(IHcaptchaInfo)
class HcaptchaInfoAnnotation(object):
    def __init__(self):
        self.error = None
        self.verified = False


HcaptchaInfo = factory(HcaptchaInfoAnnotation)


class HcaptchaView(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request
        registry = queryUtility(IRegistry)
        self.settings = registry.forInterface(IHCaptchaSettings)

    def image_tag(self):
        if not self.settings.public_key:
            return """No hcaptcha public key / site key configured.
                Go to <a href="{}/@@hcaptcha-settings" target=_blank>
                Hcaptcha Settings</a> to configure.""".format(
                getSite().absolute_url()
            )  # noqa: E501
        lang = self.request.get("LANGUAGE", "en")
        return displayhtml(
            self.settings.public_key,
            language=lang,
            theme=self.settings.display_theme,
            d_type=self.settings.display_type,
            size=self.settings.display_size,
        )

    def audio_url(self):
        return None

    def verify(self, input=None):
        info = IHcaptchaInfo(self.request)
        if info.verified:
            return True

        if not self.settings.private_key:
            raise ValueError(
                "No hcaptcha private key / secret key configured. Go to "
                "path/to/site/@@hcaptcha-settings to configure."
            )
        response_field = self.request.get("h-captcha-response")
        remote_addr = self.request.get("HTTP_X_FORWARDED_FOR", "").split(",")[0]
        if not remote_addr:
            remote_addr = self.request.get("REMOTE_ADDR")
        res = submit(response_field, self.settings.private_key, remote_addr)
        if res.error_code:
            info.error = res.error_code

        info.verified = res.is_valid
        return res.is_valid

    @property
    def external(self):
        return True



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.hcaptcha.testing import PLONE_FORMWIDGET_HCAPTCHA_INTEGRATION_TESTING  # noqa: E501

import plone.api
import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class TestSetup(unittest.TestCase):
    """Test that plone.formwidget.hcaptcha is properly installed."""

    layer = PLONE_FORMWIDGET_HCAPTCHA_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = plone.api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if plone.formwidget.hcaptcha is installed."""
        self.assertTrue(self.installer.isProductInstalled("plone.formwidget.hcaptcha"))

    def test_browserlayer(self):
        """Test that IHCaptchaLayer is registered."""
        from plone.formwidget.hcaptcha.interfaces import IHCaptchaLayer
        from plone.browserlayer import utils

        self.assertIn(IHCaptchaLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = PLONE_FORMWIDGET_HCAPTCHA_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = plone.api.portal.get_tool("portal_quickinstaller")
        roles_before = plone.api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.uninstallProducts(["plone.formwidget.hcaptcha"])
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if plone.formwidget.hcaptcha is cleanly uninstalled."""
        self.assertFalse(
            self.installer.isProductInstalled("plone.formwidget.hcaptcha")
        )

    def test_browserlayer_removed(self):
        """Test that IHCaptchaLayer is removed."""
        from plone.formwidget.hcaptcha.interfaces import IHCaptchaLayer
        from plone.browserlayer import utils

        self.assertNotIn(IHCaptchaLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility


def install_browserlayer(context):
    setup = getToolByName(context, "portal_setup")
    setup.runImportStepFromProfile(
        "profile-plone.formwidget.hcaptcha:default",
        "browserlayer",
        run_dependencies=False,
        purge_old=False,
    )


def reapply_registry(context):
    setup = getToolByName(context, "portal_setup")
    setup.runImportStepFromProfile(
        "profile-plone.formwidget.hcaptcha:default",
        "plone.app.registry",
        run_dependencies=False,
        purge_old=False,
    )


def to_4(context):

    jstool = getToolByName(context, "portal_javascripts", None)
    if jstool:
        jstool.manage_removeScript(
            "++resource++plone.formwidget.hcaptcha/hcaptcha_ajax.js"
        )

    registry = getUtility(IRegistry)
    record = "plone.bundles/plone-legacy.resources"
    if record in registry.records:
        # Plone 5
        resources = registry.records[record]
        res = "resource-plone-formwidget-hcaptcha-hcaptcha_ajax"
        if res in resources.value:
            resources.value.remove(res)

        if res in registry.records:
            del registry.records["plone.resources/%s" % res]



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.formwidget.hcaptcha.widget import HCaptchaFieldWidget
from plone.z3cform.layout import wrap_form
from z3c.form import button
from z3c.form import field
from z3c.form import form
from zope import interface
from zope import schema
from zope.component import getMultiAdapter

import logging


logger = logging.getLogger(__name__)


class IHCaptchaForm(interface.Interface):
    subject = schema.TextLine(title=u"Subject", description=u"", required=True)

    captcha = schema.TextLine(title=u"HCaptcha", description=u"", required=False)


class HCaptcha(object):
    subject = u""
    captcha = u""

    def __init__(self, context):
        self.context = context


class BaseForm(form.Form):
    """ example captcha form """

    fields = field.Fields(IHCaptchaForm)
    fields["captcha"].widgetFactory = HCaptchaFieldWidget

    @button.buttonAndHandler(u"Save")
    def handleApply(self, action):
        data, errors = self.extractData()
        captcha = getMultiAdapter(
            (aq_inner(self.context), self.request), name="hcaptcha"
        )
        if captcha.verify():
            logger.info("HCaptcha validation passed.")
        else:
            logger.info("The code you entered was wrong, please enter the new one.")
        return


HCaptchaForm = wrap_form(BaseForm)



# -*- coding: utf-8 -*-
from plone.formwidget.hcaptcha.validator import HCaptchaValidator  # noqa
from plone.formwidget.hcaptcha.widget import HCaptchaFieldWidget  # noqa
from plone.formwidget.hcaptcha.widget import HCaptchaWidget  # noqa



# -*- coding: utf-8 -*-
from plone.app.registry.browser import controlpanel
from plone.formwidget.hcaptcha.interfaces import _
from plone.formwidget.hcaptcha.interfaces import IHCaptchaSettings


class HCaptchaSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IHCaptchaSettings
    label = _(u"HCaptcha settings")
    description = _(
        u"In order to use HCaptcha on your Plone site, go to "
        u"https://www.hcaptcha.com/ to create an account and "
        u"to receive your private/secret and public/site key. Then configure them at "
        u"https://host/path/to/site/@@hcaptcha-settings. If you don't want to "
        u"rely on an external service for captcha, you might want to consider "
        u"using plone.formwidget.captcha instead."
    )

    def updateFields(self):
        super(HCaptchaSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(HCaptchaSettingsEditForm, self).updateWidgets()


class HCaptchaSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = HCaptchaSettingsEditForm



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from z3c.form import validator
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory
from zope.schema import ValidationError


_ = MessageFactory("plone.formwidget.hcaptcha")


class WrongCaptchaCode(ValidationError):
    __doc__ = _(u"The code you entered was wrong, please enter the new one.")


class HCaptchaValidator(validator.SimpleFieldValidator):
    def validate(self, value):
        super(HCaptchaValidator, self).validate(value)
        captcha = getMultiAdapter(
            (aq_inner(self.context), self.request), name="hcaptcha"
        )
        if not captcha.verify():
            raise WrongCaptchaCode
        return True



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = "1.0.1"

description = open("README.rst").read() + "\n"
description += open("CHANGES.rst").read()

setup(
    name="plone.formwidget.hcaptcha",
    version=version,
    description="Hcaptcha widget for Plone.",
    long_description=description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="plone discussion plone.app.discussion spam captcha hcaptcha",
    author="Timo Stollenwerk - Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/plone.formwidget.hcaptcha",
    license="GPLv2",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["plone", "plone.formwidget"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.app.registry",
        "plone.registry",
        "Products.CMFCore",
        "setuptools",
        "z3c.form",
        "zope.annotation",
        "zope.component",
        "zope.i18nmessageid ",
        "zope.interface",
        "zope.publisher",
        "zope.schema",
        "Zope2",
    ],
    extras_require={"test": ["plone.api", "plone.app.testing",],},
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.formwidget.hcaptcha.validator import WrongCaptchaCode
from plone.supermodel.exportimport import ObjectHandler
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.schema import Field
from zope.schema.interfaces import IField


class ICaptchaField(IField):
    """Field containing a captcha."""


@implementer(ICaptchaField)
class CaptchaField(Field):
    def get(self, object):
        # Captcha field should never save anything in the object itself
        return None

    def set(self, object, value):
        # Captcha field should never save anything in the object itself
        return None

    def validate(self, value):
        captcha = getMultiAdapter(
            (aq_inner(self.context), self.context.REQUEST), name="hcaptcha"
        )
        if not captcha.verify():
            raise WrongCaptchaCode
        return True


CaptchaHandler = ObjectHandler(CaptchaField)



# -*- coding: utf-8 -*-
from z3c.form import interfaces
from zope import schema
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


_ = MessageFactory("plone.formwidget.hcaptcha")

display_themes = SimpleVocabulary(
    [
        SimpleTerm(value=u"light", title=_(u"light")),
        SimpleTerm(value=u"dark", title=_(u"dark")),
    ]
)
display_types = SimpleVocabulary(
    [
        SimpleTerm(value=u"image", title=_(u"image")),
        SimpleTerm(value=u"audio", title=_(u"audio")),
    ]
)
display_sizes = SimpleVocabulary(
    [
        SimpleTerm(value=u"normal", title=_(u"normal")),
        SimpleTerm(value=u"compact", title=_(u"compact")),
    ]
)


class IHCaptchaLayer(Interface):
    """Browser layer for plone.formwdiget.hcaptcha"""


class IHCaptchaWidget(interfaces.IWidget):
    """Marker interface for the HCaptcha widget
    """


class IHCaptchaSettings(Interface):
    """Global discussion settings.

    This describes records stored in the configuration registry and
    obtainable via plone.registry.
    """

    # Todo: Write a short hint, that other discussion related options can
    # be found elsewhere in the Plone control panel:
    #
    # - Types control panel: Allow comments on content types
    # - Search control panel: Show comments in search results

    public_key = schema.TextLine(
        title=_(u"Public Key / Site Key"), description=_(u""), required=True, default=u""
    )

    private_key = schema.TextLine(
        title=_(u"Private Key / Secret Key"), description=_(u""), required=True, default=u""
    )

    display_theme = schema.Choice(
        title=_(u"Theme"),
        description=_(u"The color theme of the widget."),
        required=True,
        default=u"light",
        vocabulary=display_themes,
    )

    display_type = schema.Choice(
        title=_(u"Type"),
        description=_(u"The type of CAPTCHA to serve."),
        required=True,
        default=u"image",
        vocabulary=display_types,
    )

    display_size = schema.Choice(
        title=_(u"Size"),
        description=_(u"The size of the widget."),
        required=True,
        default=u"normal",
        vocabulary=display_sizes,
    )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from plone.formwidget.hcaptcha.interfaces import IHCaptchaWidget
from z3c.form import interfaces
from z3c.form import widget
from z3c.form.browser import text
from zope.component import getMultiAdapter
from zope.interface import implementer_only

import zope.component
import zope.interface
import zope.schema.interfaces


@implementer_only(IHCaptchaWidget)
class HCaptchaWidget(text.TextWidget):
    maxlength = 7
    size = 8

    def captchaImage(self):
        self.captcha = getMultiAdapter(
            (aq_inner(self.context), self.request), name="captcha"
        )
        return self.captcha.image_tag()

    def captchaAudio(self):
        self.captcha = getMultiAdapter(
            (aq_inner(self.context), self.request), name="captcha"
        )
        return self.captcha.audio_url()


@zope.component.adapter(zope.schema.interfaces.IField, interfaces.IFormLayer)
@zope.interface.implementer(interfaces.IFieldWidget)
def HCaptchaFieldWidget(field, request):
    """IFieldWidget factory for CaptchaWidget."""
    return widget.FieldWidget(field, HCaptchaWidget(request))
